# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import python_2_unicode_compatible

import os

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from ckeditor.fields import RichTextField
from easy_thumbnails.files import get_thumbnailer
from image_cropping import ImageRatioField

from jmb.core.db.models import StatusModel, DateModel, UserModel, OrderedModel
from thux.i18n.models import Language


def get_upload_small_path(instance, filename):
    media_dir = instance.slug[0:3]
    ext = filename.split('.')[-1]
    filename = '{}.{}'.format(instance.slug, ext)
    return os.path.join('item', 'small', '{}'.format(media_dir), filename)


def get_upload_big_path(instance, filename):
    media_dir = instance.slug[0:3]
    ext = filename.split('.')[-1]
    filename = '{}.{}'.format(instance.slug, ext)
    return os.path.join('item', 'big', '{}'.format(media_dir), filename)


def get_upload_file_path(instance, filename):
    media_dir = instance.slug[0:3]
    ext = filename.split('.')[-1]
    filename = '{}.{}'.format(instance.slug, ext)
    return os.path.join('item', 'file', '{}'.format(media_dir), filename)


@python_2_unicode_compatible
class Category(StatusModel, DateModel, UserModel, OrderedModel):
    title = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_('title'),
    )
    slug = models.SlugField(
        unique=True,
        max_length=255,
        verbose_name=_('slug'),
    )

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')
        ordering = ('ordering', 'title',)

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class Item(StatusModel, DateModel, UserModel, OrderedModel):
    categories = models.ManyToManyField(
        Category,
        db_index=True,
        related_name='items',
        verbose_name=_('categories'),
    )
    tale = models.CharField(
        max_length=255,
        blank=True,
        default=settings.SHOWCASE_ITEM_TALE_DEFAULT,
        verbose_name=_('tale')
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('title'),
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_('slug')
    )
    publication_date = models.DateField(
        default=timezone.now,
        verbose_name=_('publication date'),
    )
    author = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_('author'),
    )
    source = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('source'),
    )
    language = models.ForeignKey(
        Language,
        blank=True, null=True,
        verbose_name=_('language'),
    )
    content = RichTextField(
        blank=True,
        verbose_name=_('content'),
    )
    file = models.FileField(
        blank=True, null=True,
        upload_to=get_upload_file_path,
        verbose_name=_('file'),
    )
    small_image = models.ImageField(
        upload_to=get_upload_small_path,
        blank=True, null=True,
        verbose_name=_('small image')
    )
    big_image = models.ImageField(
        upload_to=get_upload_big_path,
        blank=True, null=True,
        verbose_name=_('big image')
    )
    small_cropping = ImageRatioField(
        'small_image',
        settings.SHOWCASE_ITEM_SMALL_CROPPING_SIZE,
        size_warning=True,
        verbose_name=_('small cropping'),
    )
    big_cropping = ImageRatioField(
        'big_image',
        settings.SHOWCASE_ITEM_BIG_CROPPING_SIZE,
        size_warning=True,
        verbose_name=_('big cropping'),
    )
    link = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('link'),
        help_text=_('if internal use relative link')
    )
    linkedin_link = models.URLField(
        blank=True,
        verbose_name=_('linkedin Link'),
    )
    twitter_link = models.URLField(
        blank=True,
        verbose_name=_('twitter link'),
    )

    class Meta:
        verbose_name = _('item')
        verbose_name_plural = _('items')
        unique_together = (('language', 'title',), ('language', 'slug',),)
        ordering = ('title', '-publication_date')

    def __str__(self):
        return self.title

    def get_categories(self):
        return ', '.join([category.title for category in self.categories.all().order_by('ordering')])
    get_categories.short_description = _('Categories')

    def get_small_cropping(self):
        size_list = settings.SHOWCASE_ITEM_SMALL_CROPPING_SIZE.split('x')
        for i in range(0, len(size_list)):
            size_list[i] = int(size_list[i])
        if self.small_image:
            return get_thumbnailer(
                self.small_image
            ).get_thumbnail({
                'size': tuple(size_list),
                'box': self.small_cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return None

    def get_big_cropping(self):
        size_list = settings.SHOWCASE_ITEM_BIG_CROPPING_SIZE.split('x')
        for i in range(0, len(size_list)):
            size_list[i] = int(size_list[i])
        if self.big_image:
            return get_thumbnailer(
                self.big_image
            ).get_thumbnail({
                'size': tuple(size_list),
                'box': self.big_cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return None

    def get_url_file(self):
        if self.file:
            return self.file.url
        return None
